from typing import Any

from braintree.search import Search as Search

class CustomerSearch:
    address_extended_address: Any
    address_first_name: Any
    address_last_name: Any
    address_locality: Any
    address_postal_code: Any
    address_region: Any
    address_street_address: Any
    address_country_name: Any
    cardholder_name: Any
    company: Any
    created_at: Any
    credit_card_expiration_date: Any
    credit_card_number: Any
    email: Any
    fax: Any
    first_name: Any
    id: Any
    ids: Any
    last_name: Any
    payment_method_token: Any
    payment_method_token_with_duplicates: Any
    phone: Any
    website: Any
    paypal_account_email: Any
