from datetime import date, datetime, time, timedelta

from MySQLdb._mysql import string_literal as string_literal

Date = date
Time = time
TimeDelta = timedelta
Timestamp = datetime
DateTimeDeltaType = timedelta
DateTimeType = datetime

def DateFromTicks(ticks): ...
def TimeFromTicks(ticks): ...
def TimestampFromTicks(ticks): ...

format_TIME = str
format_DATE = str

def format_TIMEDELTA(v): ...
def format_TIMESTAMP(d): ...
def DateTime_or_None(s): ...
def TimeDelta_or_None(s): ...
def Time_or_None(s): ...
def Date_or_None(s): ...
def DateTime2literal(d, c): ...
def DateTimeDelta2literal(d, c): ...
