from typing import Any, Type

def assert_(condition: object) -> None: ...

ERR_FRAGMENT: str

class AuthorizationError(Exception):
    url: Any
    reason: str

class DAVError(Exception): ...
class PropsetError(DAVError): ...
class ProppatchError(DAVError): ...
class PropfindError(DAVError): ...
class ReportError(DAVError): ...
class MkcolError(DAVError): ...
class MkcalendarError(DAVError): ...
class PutError(DAVError): ...
class DeleteError(DAVError): ...
class NotFoundError(DAVError): ...
class ConsistencyError(DAVError): ...
class ReponseError(DAVError): ...

exception_by_method: dict[str, Type[DAVError]]
