class AbstractEventListener:
    def before_navigate_to(self, url, driver) -> None: ...
    def after_navigate_to(self, url, driver) -> None: ...
    def before_navigate_back(self, driver) -> None: ...
    def after_navigate_back(self, driver) -> None: ...
    def before_navigate_forward(self, driver) -> None: ...
    def after_navigate_forward(self, driver) -> None: ...
    def before_find(self, by, value, driver) -> None: ...
    def after_find(self, by, value, driver) -> None: ...
    def before_click(self, element, driver) -> None: ...
    def after_click(self, element, driver) -> None: ...
    def before_change_value_of(self, element, driver) -> None: ...
    def after_change_value_of(self, element, driver) -> None: ...
    def before_execute_script(self, script, driver) -> None: ...
    def after_execute_script(self, script, driver) -> None: ...
    def before_close(self, driver) -> None: ...
    def after_close(self, driver) -> None: ...
    def before_quit(self, driver) -> None: ...
    def after_quit(self, driver) -> None: ...
    def on_exception(self, exception, driver) -> None: ...
