"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class DoubleValue(google.protobuf.message.Message):
    """Wrapper message for `double`.

    The JSON representation for `DoubleValue` is JSON number.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.float = ...
    """The double value."""

    def __init__(self,
        *,
        value : builtins.float = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___DoubleValue = DoubleValue

class FloatValue(google.protobuf.message.Message):
    """Wrapper message for `float`.

    The JSON representation for `FloatValue` is JSON number.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.float = ...
    """The float value."""

    def __init__(self,
        *,
        value : builtins.float = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___FloatValue = FloatValue

class Int64Value(google.protobuf.message.Message):
    """Wrapper message for `int64`.

    The JSON representation for `Int64Value` is JSON string.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.int = ...
    """The int64 value."""

    def __init__(self,
        *,
        value : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___Int64Value = Int64Value

class UInt64Value(google.protobuf.message.Message):
    """Wrapper message for `uint64`.

    The JSON representation for `UInt64Value` is JSON string.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.int = ...
    """The uint64 value."""

    def __init__(self,
        *,
        value : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___UInt64Value = UInt64Value

class Int32Value(google.protobuf.message.Message):
    """Wrapper message for `int32`.

    The JSON representation for `Int32Value` is JSON number.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.int = ...
    """The int32 value."""

    def __init__(self,
        *,
        value : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___Int32Value = Int32Value

class UInt32Value(google.protobuf.message.Message):
    """Wrapper message for `uint32`.

    The JSON representation for `UInt32Value` is JSON number.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.int = ...
    """The uint32 value."""

    def __init__(self,
        *,
        value : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___UInt32Value = UInt32Value

class BoolValue(google.protobuf.message.Message):
    """Wrapper message for `bool`.

    The JSON representation for `BoolValue` is JSON `true` and `false`.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.bool = ...
    """The bool value."""

    def __init__(self,
        *,
        value : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___BoolValue = BoolValue

class StringValue(google.protobuf.message.Message):
    """Wrapper message for `string`.

    The JSON representation for `StringValue` is JSON string.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: typing.Text = ...
    """The string value."""

    def __init__(self,
        *,
        value : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___StringValue = StringValue

class BytesValue(google.protobuf.message.Message):
    """Wrapper message for `bytes`.

    The JSON representation for `BytesValue` is JSON string.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.bytes = ...
    """The bytes value."""

    def __init__(self,
        *,
        value : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value",b"value"]) -> None: ...
global___BytesValue = BytesValue
