from typing import Any

from selenium.webdriver.common.keys import Keys as Keys

basestring = str

def free_port(): ...
def find_connectable_ip(host, port: Any | None = ...): ...
def join_host_port(host, port): ...
def is_connectable(port, host: str = ...): ...
def is_url_connectable(port): ...
def keys_to_typing(value): ...
