from abc import ABC
from typing import Any

class Action(ABC):
    def dict_as_string(self) -> None: ...

class NamedAction(Action):
    action_name: Any
    def __init__(self, action_name) -> None: ...
    def dict_as_string(self): ...

class GoToAction(Action):
    dest: Any
    def __init__(self, dest) -> None: ...
    def dict_as_string(self): ...

class GoToRemoteAction(Action):
    file: Any
    dest: Any
    def __init__(self, file, dest) -> None: ...
    def dict_as_string(self): ...

class LaunchAction(Action):
    file: Any
    def __init__(self, file) -> None: ...
    def dict_as_string(self): ...
