from typing import Any

class Options:
    @classmethod
    def options(cls, wrapped): ...
    @classmethod
    def slots(cls): ...
    def __init__(self, config: Any | None = ..., **kwargs) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def copy(self, **kwargs): ...
    def items(self) -> None: ...

class ReadOptions(Options):
    def __init__(self, config: Any | None = ..., **kwargs) -> None: ...
