from typing import Any

URL: str
METHOD: str
USER_AGENT: str
CLIENT_IP: str
X_FORWARDED_FOR: str
STATUS: str
CONTENT_LENGTH: str
XRAY_HEADER: str
ALT_XRAY_HEADER: str
request_keys: Any
response_keys: Any
