class Options:
    def __init__(self) -> None: ...
    @property
    def page_load_strategy(self): ...
    @page_load_strategy.setter
    def page_load_strategy(self, value) -> None: ...
    @property
    def capabilities(self): ...
    def set_capability(self, name, value) -> None: ...
    def to_capabilities(self): ...
