from typing import Any

from selenium.webdriver.remote.webelement import WebElement as WebElement

from . import interaction as interaction
from .interaction import Interaction as Interaction
from .mouse_button import MouseButton as MouseButton
from .pointer_input import PointerInput as PointerInput

class PointerActions(Interaction):
    source: Any
    def __init__(self, source: Any | None = ...) -> None: ...
    def pointer_down(self, button=...) -> None: ...
    def pointer_up(self, button=...) -> None: ...
    def move_to(self, element, x: Any | None = ..., y: Any | None = ...): ...
    def move_by(self, x, y): ...
    def move_to_location(self, x, y): ...
    def click(self, element: Any | None = ...): ...
    def context_click(self, element: Any | None = ...): ...
    def click_and_hold(self, element: Any | None = ...): ...
    def release(self): ...
    def double_click(self, element: Any | None = ...) -> None: ...
    def pause(self, duration: int = ...): ...
