from typing import Text

from .en import (
    EMOJI_ALIAS_UNICODE_ENGLISH as EMOJI_ALIAS_UNICODE_ENGLISH,
    EMOJI_UNICODE_ENGLISH as EMOJI_UNICODE_ENGLISH,
    UNICODE_EMOJI_ALIAS_ENGLISH as UNICODE_EMOJI_ALIAS_ENGLISH,
    UNICODE_EMOJI_ENGLISH as UNICODE_EMOJI_ENGLISH,
)
from .es import EMOJI_UNICODE_SPANISH as EMOJI_UNICODE_SPANISH, UNICODE_EMOJI_SPANISH as UNICODE_EMOJI_SPANISH
from .it import EMOJI_UNICODE_ITALIAN as EMOJI_UNICODE_ITALIAN, UNICODE_EMOJI_ITALIAN as UNICODE_EMOJI_ITALIAN
from .pt import EMOJI_UNICODE_PORTUGUESE as EMOJI_UNICODE_PORTUGUESE, UNICODE_EMOJI_PORTUGUESE as UNICODE_EMOJI_PORTUGUESE

EMOJI_UNICODE: dict[str, dict[Text, Text]]
UNICODE_EMOJI: dict[str, dict[Text, Text]]
