LEXER_ENTRY_POINT: str
FORMATTER_ENTRY_POINT: str
STYLE_ENTRY_POINT: str
FILTER_ENTRY_POINT: str

def iter_entry_points(group_name): ...
def find_plugin_lexers() -> None: ...
def find_plugin_formatters() -> None: ...
def find_plugin_styles() -> None: ...
def find_plugin_filters() -> None: ...
