from typing import Any

PY2: Any
text_type = str
string_types: Any
integer_types: Any
text_to_native: Any
unichr = chr
iterkeys: Any
itervalues: Any
iteritems: Any
izip = zip
imap = map
range_type = range
cmp: Any
array_tobytes: Any
number_types: Any

def force_text(s, encoding: str = ..., errors: str = ...): ...
