from typing import Any

def compact(text, *args, **kw): ...
def compact_empty_lines(text): ...
def concatenate(items, conjunction: str = ..., serial_comma: bool = ...): ...
def dedent(text, *args, **kw): ...
def format(text, *args, **kw): ...
def generate_slug(text, delimiter: str = ...): ...
def is_empty_line(text): ...
def join_lines(text): ...
def pluralize(count, singular, plural: Any | None = ...): ...
def pluralize_raw(count, singular, plural: Any | None = ...): ...
def random_string(length=..., characters=...): ...
def split(text, delimiter: str = ...): ...
def split_paragraphs(text): ...
def tokenize(text): ...
def trim_empty_lines(text): ...
