from typing import Any

from stripe import api_requestor as api_requestor, error as error
from stripe.stripe_object import StripeObject as StripeObject

class APIResource(StripeObject):
    @classmethod
    def retrieve(cls, id, api_key: Any | None = ..., **params): ...
    def refresh(self): ...
    @classmethod
    def class_url(cls): ...
    def instance_url(self): ...
