from typing import Any

from .base import GrantTypeBase as GrantTypeBase

log: Any

def code_challenge_method_s256(verifier, challenge): ...
def code_challenge_method_plain(verifier, challenge): ...

class AuthorizationCodeGrant(GrantTypeBase):
    default_response_mode: str
    response_types: Any
    def create_authorization_code(self, request): ...
    def create_authorization_response(self, request, token_handler): ...
    def create_token_response(self, request, token_handler): ...
    def validate_authorization_request(self, request): ...
    def validate_token_request(self, request) -> None: ...
    def validate_code_challenge(self, challenge, challenge_method, verifier): ...
