from typing import Text

from . import Markup
from ._compat import text_type

def escape(s: Markup | Text) -> Markup: ...
def escape_silent(s: None | Markup | Text) -> Markup: ...
def soft_unicode(s: Text) -> text_type: ...
