# Test trailing comma after last arg

def h1(a,):
    # type: (a1) -> r1
    pass

def h2(a,b,):
    # type: (a2, b2) -> r2
    pass

def h3(a):
    # type: (a3) -> r3
    pass

def h4(a):
    # type: (a4) -> r4
    pass
