import types
from typing import Any

def define_aliases(module_name, **aliases) -> None: ...
def get_aliases(module_name): ...
def deprecated_args(*names): ...
def is_method(function): ...

class DeprecationProxy(types.ModuleType):
    module: Any
    aliases: Any
    def __init__(self, module, aliases) -> None: ...
    def __getattr__(self, name): ...
    def resolve(self, target): ...
