from typing import Any

class FormatChecker:
    checkers: Any
    def __init__(self, formats: Any | None = ...) -> None: ...
    def checks(self, format, raises=...): ...
    cls_checks: Any
    def check(self, instance, format) -> None: ...
    def conforms(self, instance, format): ...

draft3_format_checker: Any
draft4_format_checker: Any
draft6_format_checker: Any
draft7_format_checker: Any

def is_email(instance): ...
def is_ipv4(instance): ...
def is_ipv6(instance): ...
def is_host_name(instance): ...
def is_idn_host_name(instance): ...
def is_uri(instance): ...
def is_uri_reference(instance): ...
def is_iri(instance): ...
def is_iri_reference(instance): ...
def is_datetime(instance): ...
def is_time(instance): ...
def is_regex(instance): ...
def is_date(instance): ...
def is_draft3_time(instance): ...
def is_css_color_code(instance): ...
def is_css21_color(instance): ...
def is_css3_color(instance): ...
def is_json_pointer(instance): ...
def is_relative_json_pointer(instance): ...
def is_uri_template(instance, template_validator=...): ...
