class Command:
    STATUS: str
    NEW_SESSION: str
    GET_ALL_SESSIONS: str
    DELETE_SESSION: str
    CLOSE: str
    QUIT: str
    GET: str
    GO_BACK: str
    GO_FORWARD: str
    REFRESH: str
    ADD_COOKIE: str
    GET_COOKIE: str
    GET_ALL_COOKIES: str
    DELETE_COOKIE: str
    DELETE_ALL_COOKIES: str
    FIND_ELEMENT: str
    FIND_ELEMENTS: str
    FIND_CHILD_ELEMENT: str
    FIND_CHILD_ELEMENTS: str
    CLEAR_ELEMENT: str
    CLICK_ELEMENT: str
    SEND_KEYS_TO_ELEMENT: str
    SEND_KEYS_TO_ACTIVE_ELEMENT: str
    SUBMIT_ELEMENT: str
    UPLOAD_FILE: str
    GET_CURRENT_WINDOW_HANDLE: str
    W3C_GET_CURRENT_WINDOW_HANDLE: str
    GET_WINDOW_HANDLES: str
    W3C_GET_WINDOW_HANDLES: str
    GET_WINDOW_SIZE: str
    W3C_GET_WINDOW_SIZE: str
    W3C_GET_WINDOW_POSITION: str
    GET_WINDOW_POSITION: str
    SET_WINDOW_SIZE: str
    W3C_SET_WINDOW_SIZE: str
    SET_WINDOW_RECT: str
    GET_WINDOW_RECT: str
    SET_WINDOW_POSITION: str
    W3C_SET_WINDOW_POSITION: str
    SWITCH_TO_WINDOW: str
    SWITCH_TO_FRAME: str
    SWITCH_TO_PARENT_FRAME: str
    GET_ACTIVE_ELEMENT: str
    W3C_GET_ACTIVE_ELEMENT: str
    GET_CURRENT_URL: str
    GET_PAGE_SOURCE: str
    GET_TITLE: str
    EXECUTE_SCRIPT: str
    W3C_EXECUTE_SCRIPT: str
    W3C_EXECUTE_SCRIPT_ASYNC: str
    GET_ELEMENT_TEXT: str
    GET_ELEMENT_VALUE: str
    GET_ELEMENT_TAG_NAME: str
    SET_ELEMENT_SELECTED: str
    IS_ELEMENT_SELECTED: str
    IS_ELEMENT_ENABLED: str
    IS_ELEMENT_DISPLAYED: str
    GET_ELEMENT_LOCATION: str
    GET_ELEMENT_LOCATION_ONCE_SCROLLED_INTO_VIEW: str
    GET_ELEMENT_SIZE: str
    GET_ELEMENT_RECT: str
    GET_ELEMENT_ATTRIBUTE: str
    GET_ELEMENT_PROPERTY: str
    GET_ELEMENT_VALUE_OF_CSS_PROPERTY: str
    SCREENSHOT: str
    ELEMENT_SCREENSHOT: str
    IMPLICIT_WAIT: str
    EXECUTE_ASYNC_SCRIPT: str
    SET_SCRIPT_TIMEOUT: str
    SET_TIMEOUTS: str
    MAXIMIZE_WINDOW: str
    W3C_MAXIMIZE_WINDOW: str
    GET_LOG: str
    GET_AVAILABLE_LOG_TYPES: str
    FULLSCREEN_WINDOW: str
    MINIMIZE_WINDOW: str
    DISMISS_ALERT: str
    W3C_DISMISS_ALERT: str
    ACCEPT_ALERT: str
    W3C_ACCEPT_ALERT: str
    SET_ALERT_VALUE: str
    W3C_SET_ALERT_VALUE: str
    GET_ALERT_TEXT: str
    W3C_GET_ALERT_TEXT: str
    SET_ALERT_CREDENTIALS: str
    W3C_ACTIONS: str
    W3C_CLEAR_ACTIONS: str
    CLICK: str
    DOUBLE_CLICK: str
    MOUSE_DOWN: str
    MOUSE_UP: str
    MOVE_TO: str
    SET_SCREEN_ORIENTATION: str
    GET_SCREEN_ORIENTATION: str
    SINGLE_TAP: str
    TOUCH_DOWN: str
    TOUCH_UP: str
    TOUCH_MOVE: str
    TOUCH_SCROLL: str
    DOUBLE_TAP: str
    LONG_PRESS: str
    FLICK: str
    EXECUTE_SQL: str
    GET_LOCATION: str
    SET_LOCATION: str
    GET_APP_CACHE: str
    GET_APP_CACHE_STATUS: str
    CLEAR_APP_CACHE: str
    GET_LOCAL_STORAGE_ITEM: str
    REMOVE_LOCAL_STORAGE_ITEM: str
    GET_LOCAL_STORAGE_KEYS: str
    SET_LOCAL_STORAGE_ITEM: str
    CLEAR_LOCAL_STORAGE: str
    GET_LOCAL_STORAGE_SIZE: str
    GET_SESSION_STORAGE_ITEM: str
    REMOVE_SESSION_STORAGE_ITEM: str
    GET_SESSION_STORAGE_KEYS: str
    SET_SESSION_STORAGE_ITEM: str
    CLEAR_SESSION_STORAGE: str
    GET_SESSION_STORAGE_SIZE: str
    GET_NETWORK_CONNECTION: str
    SET_NETWORK_CONNECTION: str
    CURRENT_CONTEXT_HANDLE: str
    CONTEXT_HANDLES: str
    SWITCH_TO_CONTEXT: str
