import sys

if sys.platform != "win32":
    def cat(map: str, domain: str = ...) -> dict[str, str]: ...
    def get_default_domain() -> str: ...
    def maps(domain: str = ...) -> list[str]: ...
    def match(key: str, map: str, domain: str = ...) -> str: ...
    class error(Exception): ...
