from typing import Any, MutableMapping

class URIDict(MutableMapping[Any, Any]):
    def normalize(self, uri): ...
    store: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def __getitem__(self, uri): ...
    def __setitem__(self, uri, value) -> None: ...
    def __delitem__(self, uri) -> None: ...
    def __iter__(self): ...
    def __len__(self): ...

class Unset: ...

def load_schema(name): ...
def indent(string, times: int = ...): ...
def format_as_index(indices): ...
def find_additional_properties(instance, schema) -> None: ...
def extras_msg(extras): ...
def types_msg(instance, types): ...
def flatten(suitable_for_isinstance): ...
def ensure_list(thing): ...
def equal(one, two): ...
def unbool(element, true=..., false=...): ...
def uniq(container): ...
