from typing import Any

class BaseSerializer:
    content_types: Any
    key: Any
    def get_content_type(self): ...
    def loads(self, data) -> None: ...
    def dumps(self, data) -> None: ...

class JsonSerializer(BaseSerializer):
    content_types: Any
    key: str
    def loads(self, data): ...
    def dumps(self, data): ...

class YamlSerializer(BaseSerializer):
    content_types: Any
    key: str
    def loads(self, data): ...
    def dumps(self, data): ...

class Serializer:
    serializers: Any
    default: Any
    def __init__(self, default: Any | None = ..., serializers: Any | None = ...) -> None: ...
    def get_serializer(self, name: Any | None = ..., content_type: Any | None = ...): ...
    def loads(self, data, format: Any | None = ...): ...
    def dumps(self, data, format: Any | None = ...): ...
    def get_content_type(self, format: Any | None = ...): ...
