from typing import Any, Dict, List, Text

_ModuleGlobals = Dict[str, Any]

def getline(filename: Text, lineno: int, module_globals: _ModuleGlobals | None = ...) -> str: ...
def clearcache() -> None: ...
def getlines(filename: Text, module_globals: _ModuleGlobals | None = ...) -> List[str]: ...
def checkcache(filename: Text | None = ...) -> None: ...
def updatecache(filename: Text, module_globals: _ModuleGlobals | None = ...) -> List[str]: ...
