from .core import (
    demojize as demojize,
    emoji_count as emoji_count,
    emoji_lis as emoji_lis,
    emojize as emojize,
    get_emoji_regexp as get_emoji_regexp,
)
from .unicode_codes import (
    EMOJI_ALIAS_UNICODE_ENGLISH as EMOJI_ALIAS_UNICODE_ENGLISH,
    EMOJI_UNICODE_ENGLISH as EMOJI_UNICODE_ENGLISH,
    EMOJI_UNICODE_ITALIAN as EMOJI_UNICODE_ITALIAN,
    EMOJI_UNICODE_PORTUGUESE as EMOJI_UNICODE_PORTUGUESE,
    EMOJI_UNICODE_SPANISH as EMOJI_UNICODE_SPANISH,
    UNICODE_EMOJI_ALIAS_ENGLISH as UNICODE_EMOJI_ALIAS_ENGLISH,
    UNICODE_EMOJI_ENGLISH as UNICODE_EMOJI_ENGLISH,
    UNICODE_EMOJI_ITALIAN as UNICODE_EMOJI_ITALIAN,
    UNICODE_EMOJI_PORTUGUESE as UNICODE_EMOJI_PORTUGUESE,
    UNICODE_EMOJI_SPANISH as UNICODE_EMOJI_SPANISH,
)
