def list_to_scope(scope): ...
def scope_to_list(scope): ...
def params_from_uri(uri): ...
def host_from_uri(uri): ...
def escape(u): ...
def generate_age(issue_time): ...
def is_secure_transport(uri): ...
